/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 1998, 2000, 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * FILE NAME
 *      authzn_demo.c
 *
 * DESCRIPTION
 *      An example program that uses the Policy Director aznAPI.
 */

#include <stdlib.h>
#include <stdio.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif /* WIN32 */
#include <ctype.h>
#include "ogauthzn.h"
#include "aznutils.h"
#include "pdbaclmsg.h"

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#define LINELEN 128

#ifdef WIN32
#define STRCASECMP _stricmp
#define STRNCASECMP _strnicmp
#else
#define STRCASECMP strcasecmp
#define STRNCASECMP strncasecmp
#endif /* WIN32 */


/* debugging: dump an attr list given a handle */
void attrlist_dump(azn_attrlist_h_t );

/*
 * Writes usage message and exits
 */
void
usage(
    const char *progname
)
{
    fprintf(stderr, "\nUSAGE:\n\n   %s [ -cfgfile <file> [ -override ] ] [ -local ]\n\n", progname);
    fprintf(stderr, "where:\n");
    fprintf(stderr, "   -cfgfile:  Configure aznAPI using a config file.\n\n");
    fprintf(stderr, "   -override: Override the config file with parameters passed explicitly\n");
    fprintf(stderr, "              to azn_initialize as parameters.\n\n");
    fprintf(stderr, "   -local:   Use local policy cache (default: remote).\n\n");
    exit(1);
}


/*
 * Prompt the user for a variable.
 */
int
readVar(
    const char *prompt,
    char *var,
    int len
)
{
    char        buffer[LINELEN];
    char        *startpos;
    char        *endpos;

    /* Display prompt */
    printf("%s [%s]: ", prompt, var);
    fflush(stdout);

    /* Get user input */
    if ( fgets(buffer, (LINELEN-1), stdin) == NULL )
        return FALSE;

    buffer[LINELEN-1] = '\0';

    /* Skip leading white space */
    startpos = buffer;
    while ( *startpos && isspace(*startpos) ) {
        startpos++;
    }

    /* If empty, return the current value */
    if ( ! *startpos ) {
        return TRUE;
    }

    /* Skip trailing white space */
    endpos = startpos + strlen(startpos) - 1;
    while ( endpos != startpos && isspace(*endpos) ) {
        endpos--;
    }
    endpos[1] = '\0';

    /* check for end of exit marker */
    if (STRCASECMP(startpos, "exit") == 0)
        return FALSE;

    /* Copy the user's input to return */
    strcpy(var, startpos);
    return TRUE;
}


/*
 * check status and print a message.
 */
void
check_status(
    char *info,
    unsigned status
)
{
    azn_string_t errstr = NULL;
    unsigned majerr, minerr;
    azn_status_t rc;

    if (status == AZN_S_COMPLETE)
        return;

    majerr = azn_error_major(status);
    minerr = azn_error_minor(status);

    printf("\n%s: ", info);

    rc = azn_error_get_string(status, &errstr);
    if (rc == AZN_S_COMPLETE && errstr) {
	printf("%s (0x%08x/0x%08x)\n\n", errstr, majerr, minerr);
	azn_release_string(&errstr);
    }
    else
        printf("API failure (0x%08x/0x%08x)\n\n", majerr, minerr);
}


/*
 * Set the LDAP initialization attributes.
 */
azn_status_t
set_ldap_init_attributes(
    azn_attrlist_h_t    ldapinitdata
)
{
    unsigned short      port = 389;
    char                buf[LINELEN];
    azn_status_t        status;


    /* prompt, read and set the ldap host attribute */
    sprintf(buf, "claret");
    while ( ! readVar("\nLDAP host name:", buf, LINELEN) ) {
        fprintf(stderr, "readVar error: please retry!\n\n");
    }
    status = azn_attrlist_add_entry(ldapinitdata,
                                    azn_init_ldap_host,
                                    buf);
    check_status("add attribute ldap host", status);
    if (status != AZN_S_COMPLETE)
        return(status);


    /* prompt, read and set the ldap port attribute */
    sprintf(buf, "%d", port);
    while ( ! readVar("LDAP server port:", buf, LINELEN) ) {
        fprintf(stderr, "readVar error: please retry!\n\n");
    }
    status = azn_attrlist_add_entry(ldapinitdata,
                                    azn_init_ldap_port,
                                    buf);
    check_status("add attribute ldap port", status);
    if (status != AZN_S_COMPLETE)
        return(status);


    /* prompt, read and set the ldap bind DN attribute */
    sprintf(buf, "cn=root");
    while ( ! readVar("LDAP root DN name:", buf, LINELEN) ) {
        fprintf(stderr, "readVar error: please retry!\n\n");
    }
    status = azn_attrlist_add_entry(ldapinitdata,
                                    azn_init_ldap_bind_dn,
                                    buf);
    check_status("add attribute ldap bind dn", status);
    if (status != AZN_S_COMPLETE)
        return(status);


    /* prompt, read and set the ldap bind DN password attribute */
    sprintf(buf, "");
    while ( ! readVar("LDAP root DN password:", buf, LINELEN) ) {
        fprintf(stderr, "readVar error: please retry!\n\n");
    }
    status = azn_attrlist_add_entry(ldapinitdata,
                                    azn_init_ldap_bind_pwd,
                                    buf);
    check_status("add attribute ldap bind pwd", status);
    if (status != AZN_S_COMPLETE)
        return(status);


    /* prompt, read and set the ldap SSL key file attribute */
    sprintf(buf, "");
    while (!readVar("SSL key file <Empty for no SSL>:", buf, LINELEN)) {
        fprintf(stderr, "readVar error: please retry!\n\n");
    }
    /*
     * If the SSL key file path is empty then this indicates
     * that there is no requirement for a connection via SSL
     * to the LDAP server.
     */
    if (buf != NULL && strcmp(buf, "") != 0) {

        status = azn_attrlist_add_entry(ldapinitdata,
                                        azn_init_ldap_ssl_keyfile,
                                        buf);
        check_status("add attribute ldap ssl keyfile", status);
        if (status != AZN_S_COMPLETE)
            return(status);

        /* prompt, read and set the ldap SSL key file DN attribute */
        sprintf(buf, "");
        while ( ! readVar("SSL key file DN:", buf, LINELEN) ) {
            fprintf(stderr, "readVar error: please retry!\n\n");
        }
        status = azn_attrlist_add_entry(ldapinitdata,
                                        azn_init_ldap_ssl_keyfile_dn,
                                        buf);
        check_status("add attribute ldap ssl keyfile dn", status);
        if (status != AZN_S_COMPLETE)
            return(status);


        /* prompt, read and set the ldap SSL key file DN password attribute */
        sprintf(buf, "");
        while ( ! readVar("SSL key file password:", buf, LINELEN) ) {
            fprintf(stderr, "readVar error: please retry!\n\n");
        }
        status = azn_attrlist_add_entry(ldapinitdata,
                                        azn_init_ldap_ssl_keyfile_pwd,
                                        buf);
        check_status("add attribute ldap ssl keyfile pwd", status);
        if (status != AZN_S_COMPLETE)
            return(status);
    }

    printf("\n");

    return(AZN_S_COMPLETE);
}


/*
 * Set the AZN API initialization attributes.
 */
azn_status_t
set_init_attributes(
    azn_attrlist_h_t    azninitdata,
    int                 local_mode
)
{
    azn_status_t        status;


    if ( local_mode ) {
        /*** Use a local DB replica ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_mode,
                                        "local");
        check_status("add attribute azn_init_mode", status);
        if (status != AZN_S_COMPLETE)
            return(status);

        /*** The file name of the replica policy database ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_db_file,
                                        "./auth_demo.db");
        check_status("add attribute azn_init_db_file", status);
        if (status != AZN_S_COMPLETE)
            return(status);

        /*** The file name of the audit file ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_audit_file,
                                        "./auth_demo.audit");
        check_status("add attribute azn_init_audit_file", status);
        if (status != AZN_S_COMPLETE)
            return(status);

        /*** Enable polled updates at the default interval ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_cache_refresh_interval,
                                        "default");
        check_status("add attribute azn_init_cache_refresh_interval", status);
        if (status != AZN_S_COMPLETE)
            return(status);

        /*** Enable the update notification listener ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_listen_flags,
                                        "enable");
        check_status("add attribute azn_init_listen_flags: enable", status);
        if (status != AZN_S_COMPLETE)
            return(status);

    }
    else {
        /*** Don't use a local replica, use the authorization server ***/

        status = azn_attrlist_add_entry(azninitdata,
                                        azn_init_mode,
                                        "remote");
        check_status("add attribute azn_init_mode", status);
        if (status != AZN_S_COMPLETE)
            return(status);

    }

    return(AZN_S_COMPLETE);
}


/*
 *  Prompt the user for the principal, object and permission to check for.
 *  Perform an authorization check and print the result to stdout.
 */
int
main(
    int argc,
    char **argv
)
{
    azn_status_t                status;
    int                         permitted;
    int                         local = FALSE;
    int                         isServer = FALSE;
    int                         isClient = FALSE;
    int                         haveCfgFile = FALSE;
    int                         confOverride = FALSE;
    int                         x;
    char                        testuser[LINELEN];
    char                        testuserpwd[LINELEN];
    char                        prin_name[LINELEN];
    char                        operation[LINELEN];
    char                        obj_name[LINELEN];
    char                        cfgFile[LINELEN];
    azn_unauth_t                unauth_minfo;
    azn_authdefault_t           default_minfo;
    azn_creds_h_t               creds = AZN_C_INVALID_HANDLE;
    azn_attrlist_h_t            initdata = AZN_C_INVALID_HANDLE;
    azn_attrlist_h_t            initinfo = AZN_C_INVALID_HANDLE;
    azn_string_t                *attrnames;


    /* terminate the input strings */
    testuser[0] = (char)0;
    testuserpwd[0] = (char)0;
    prin_name[0] = (char)0;
    operation[0] = (char)0;
    obj_name[0] = (char)0;
    cfgFile[0] = (char)0;


    /*
     * Parse command-line arguments
     */
    printf("\n");
    for (x=1; x < argc; x++) {
        if ( STRCASECMP(argv[x], "-cfgfile") == 0 ) {
            x++;
            if (x >= argc || argv[x][0] == '-') {
                fprintf(stderr,
                        "\n-cfgfile needs config file pathname parameter!\n\n");
                usage(argv[0]);
            }
            sprintf(cfgFile, "%s", argv[x]);
            if (strlen(cfgFile) <= 0) {
                fprintf(stderr,
                        "\nInvalid config file pathname parameter!\n\n");
                usage(argv[0]);
            }
	    haveCfgFile = TRUE;
        }
	else if ( STRCASECMP(argv[x], "-override") == 0 ) {
            confOverride = TRUE;
        }
	else if ( STRCASECMP(argv[x], "-local") == 0 ) {
            local = TRUE;
            }
        else {
                usage(argv[0]);
            }
        }

    /* ensure we have a conf file to override */
    if (confOverride && !haveCfgFile) {
	fprintf(stderr, "\n-override cannot be specified without -cfgfile!\n\n");
	usage(argv[0]);
    }

    /* Create handles to attribute lists */
    azn_attrlist_create(&initdata);

    /* print some informational trace */
    if (local)
        printf("** Configuring local replica **\n");
    else
        printf("** Using remote replica (ivacld) **\n");


    /* initialise the LDAP registry interface */
    if (!haveCfgFile || confOverride) {
        if (set_ldap_init_attributes(initdata) != AZN_S_COMPLETE)
            exit(-1);
    }


    /* default creds mechanism info */
    default_minfo.auth_method = "default_auth_method";
    default_minfo.authnmech_info = "default_authnmech_info";
    default_minfo.qop = "default_qop";
    default_minfo.user_info = "default_user_info";
    default_minfo.browser_info = "default_browser_info";
    default_minfo.ipaddr = 0x0a000002;

    /* unauth creds */
    unauth_minfo.qop = "unauth_qop";
    unauth_minfo.user_info = "unauth_user_info";
    unauth_minfo.browser_info = "unauth_browser_info";
    unauth_minfo.ipaddr = 0x0a0000ff;



    /*
     * Build startup attribute list. If a config file is
     * given then load this first. If not then load the
     * azn_mod_rad service ourselves.
     */
    if (haveCfgFile) {
	status = azn_attrlist_add_entry(initdata,
					azn_init_cfg_file,
					cfgFile);
	check_status("add attribute azn_init_cfg_file", status);
	if (status != AZN_S_COMPLETE)
	    exit(status);
    }


#if OPTIONAL_EXAMPLE_API
    /*
     * For extended version decision call: Add attribute
     * to activate the return of all available permission
     * info attributes for the calls. Without specifying
     * this attribute the extended version decision call
     * will not return any permission info. The attribute
     * to add is: azn_init_set_perminfo_attrs. The values
     * that this attribute can have are any of the
     * azn_perminfo_* attributes defined in ogauthzn.h.
     * If 'azn_perminfo_all_attrs' is passed as a value
     * then the call will return all available permission
     * info attributes for the decision.
     */
	    status = azn_attrlist_add_entry(initdata,
					    azn_init_set_perminfo_attrs,
				    azn_perminfo_all_attrs);
	    check_status("add attribute azn_init_set_perminfo_attrs", status);
	    if (status != AZN_S_COMPLETE)
		exit(status);

#endif


    /*
     * We load the configuration programmatically if there is
     * not conf file or the confOverride flag has been set. In
     * the case of the latter adding attributes to the init_data
     * attribute list along with a conf file will automatically
     * override any corresponding attributes set in the conf file
     * with those passed explicitly in the init_data attr list.
     */
    if (!haveCfgFile || confOverride) {
	status = set_init_attributes(initdata, local);
	if (status != AZN_S_COMPLETE)
	    exit(status);
    }


    /* view the attribute list before initializing */
    status = azn_attrlist_get_names(initdata, &attrnames);
    if ( status != AZN_S_COMPLETE ) {
	check_status("azn_attrlist_get_names", status);
        exit (status);
    }

    printf("\n[[ Init attribute list ]]\n");
    for (x=0; attrnames[x] != NULL; x++)
        printf("    [%02d]: %s\n", x, attrnames[x]);
    printf("\n");

    azn_release_strings(&attrnames);

    /* Start the service */
    status = azn_initialize(initdata, &initinfo);
    check_status("azn_initialize", status);
    if (status != AZN_S_COMPLETE) {
        fprintf(stderr,
                "Could not initialise the authorization service!\n", status);
	if (azn_util_handle_is_valid(initinfo)) {
	    printf("\n[[ Attribute list returned from azn_initialize ]]\n");
	    attrlist_dump(initinfo);
	}
        exit(1);
    }

    /*
     * View the returned initinfo attribute list including
     * the AZN API client lib version.
     */
    if (initinfo != AZN_C_INVALID_HANDLE) {
	status = azn_attrlist_get_names(initinfo, &attrnames);
	if ( status == AZN_S_COMPLETE ) {
	    printf("\n[[ Attribute list returned from azn_initialize ]]\n");
	    for (x=0; attrnames[x] != NULL; x++)
		printf("    [%02d]: %s\n", x, attrnames[x]);
	    printf("\n");

	    azn_release_strings(&attrnames);
	}
	else
	    check_status("azn_attrlist_get_names", status);
    }

    /* delete handles to attribute lists */
    azn_attrlist_delete(&initdata);
    azn_attrlist_delete(&initinfo);

        /*
     * Prompt the user for a principal name, object name and permission name
     */
    strcpy(prin_name, "unauthenticated");
    strcpy(operation, azn_operation_read);
    strcpy(obj_name, "/Management");

    while ( TRUE ) {
        azn_string_t    mech = NULL;
        azn_buffer_desc buf = { 0, 0};
        int             dobreak = FALSE;

        printf("\n-- Type \"exit\" at any prompt to exit this program.\n\n");

        do {
            /* Get principal to check for */
            if ( ! readVar("Principal", prin_name, LINELEN) ) {
                dobreak = TRUE;
                break;
            }

            /*
            * An example of calling password authenticate.
            * Not required for the authzn_demo. This does
            * not affect the azn_id_get_creds() call that
            * follows. The returned structure info is simply
            * printed out as an example. In a real usage
            * situation the info structure returned would
            * be modified as required then passed directly
            * on to azn_id_get_creds().
            */
#if OPTIONAL_EXAMPLE_API
            if ( STRNCASECMP(prin_name, "unauth", 6) != 0 ) {
                azn_string_t mechidp;
                azn_buffer_t bufp = NULL;

                /* Get password to login with */
                testuserpwd[0] = (char) 0;
                if ( ! readVar("Password", testuserpwd, LINELEN) ) {
		    dobreak = TRUE;
		    break;
                }

                /* call password authenticate */
                status = azn_util_password_authenticate(prin_name,
                                                        testuserpwd,
                                                        &mechidp,
                                                        &bufp);
                check_status("azn_util_password_authenticate", status);
                if (status != AZN_S_COMPLETE) {
                    fprintf(stderr, "Password authenticate failed!\n\n");
                }
                else {
                    if (strcmp(IV_LDAP, mechidp) == 0)
                        printf("Password authenticate LDAP: DN='%s'.\n\n",
                                        ((azn_authldap_t *)bufp->value)->ldap_dn);
                    else if (strcmp(IV_URAF, mechidp) == 0)
                        printf("Password authenticate URAF: UserID='%s'.\n\n",
                                        ((azn_authuraf_t *)bufp->value)->uraf_user);
                    else
                        fprintf(stderr, "Invalid mechanism ID: '%s'!\n\n",
                                    mechidp);
                }

                azn_release_string(&mechidp);
                azn_release_buffer(&bufp);
            }
#endif

            /* build a mech info structures */
            if ( STRNCASECMP(prin_name, "unauth", 6) == 0 ) {
                mech = (char *)IV_UNAUTH;
                buf.length = sizeof(unauth_minfo);
                buf.value = (unsigned char *)&unauth_minfo;
            }
            else {
                /*
                 * Set mechanism ID to NULL so the aznAPI will determine
                 * which registry (LDAP or URAF) that Policy Director is
                 * configured to use.
                 */
                mech = (char *)NULL;
                default_minfo.principal = prin_name;
                buf.length = sizeof(default_minfo);
                buf.value = (unsigned char *)&default_minfo;
            }

            /* create a cred */
            status = azn_creds_create(&creds);
            check_status("azn_creds_create", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could not create a cred!\n");
                exit(1);
            }

            /* get a cred */
            status = azn_id_get_creds(NULL, mech, &buf, &creds);
            check_status("azn_id_get_creds", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could not get creds!\n\n");
            }
        } while (status != AZN_S_COMPLETE);

        /* check if the user wants to break out */
        if (dobreak == TRUE)
            break;


        /*
         * an example usage of PAC'ing and unPAC'ing a cred.
         * Not required for the authzn_demo.
         */
#if OPTIONAL_EXAMPLE_API
        {
            azn_buffer_t                bufp;
            azn_creds_h_t               newcreds = AZN_C_INVALID_HANDLE;

            status = azn_creds_get_pac(creds, NULL, &bufp);
            check_status("azn_creds_get_pac", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could not PAC creds!\n\n");
            }

            status = azn_creds_create(&newcreds);
            check_status("azn_creds_create", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could not create newcreds!\n");
                exit(1);
            }

            status = azn_pac_get_creds(bufp, NULL, &newcreds);
            check_status("azn_pac_get_creds", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could not un-PAC creds!\n\n");
            }

            status = azn_creds_delete(&newcreds);
            status = azn_release_buffer(&bufp);
        }
#endif


        /*
         * An example of calling azn_creds_get_attrlist_for_subject.
         * Not required for the authzn_demo.
         */
#if OPTIONAL_EXAMPLE_API
        {
            azn_attrlist_h_t    cred_attrs = AZN_C_INVALID_HANDLE;

            status = azn_creds_get_attrlist_for_subject(creds,
                                                        AZN_C_INITIATOR_INDEX,
                                                        &cred_attrs);
            check_status("azn_creds_get_attrlist_for_subject", status);

            /*
             * dump the attrlist if successful
             */
            if (status == AZN_S_COMPLETE)
                attrlist_dump(cred_attrs);
            else
                fprintf(stderr, "Could not get attrlist for creds!\n\n");

            azn_attrlist_delete(&cred_attrs);
        }
#endif

        printf( "\nPlease build an action string from the following.\n"
                "       [a] azn_operation_attach\n"
                "       [b] azn_operation_browse\n"
                "       [B] azn_operation_bypasstod\n"
                "       [c] azn_operation_control\n"
                "       [T] azn_operation_traverse\n"
                "       [g] azn_operation_delegation\n"
                "       [v] azn_operation_view\n"
                "       [m] azn_operation_modify\n"
                "       [d] azn_operation_delete\n"
                "       [s] azn_operation_server_admin\n"
                "       [r] azn_operation_read\n"
                "       [x] azn_operation_execute\n"
                "       [l] azn_operation_list_directory\n"
                "       [W] azn_operation_password\n"
                "       [N] azn_operation_create\n"
                "       [A] azn_operation_add\n\n");

        /* Get action to check for */
        if ( ! readVar("Action(s)", operation, LINELEN) ) {
	    dobreak = TRUE;
            break;
        }

        /* Get target object */
        if ( ! readVar("Object", obj_name, LINELEN) ) {
	    dobreak = TRUE;
            break;
        }


#if ! defined(OPTIONAL_EXAMPLE_API)

        /* Perform standard authorization check */
        status = azn_decision_access_allowed(creds,
                                             obj_name,
                                             operation,
                                             &permitted);
        check_status("azn_decision_access_allowed", status);


#else /* OPTIONAL_EXAMPLE_API */

        /*
         * An example of calling azn_decision_access_allowed_ext()
         * Not required for the basic authzn_demo. Demonstrates
         * the use of permission info attribute list. Returns no
         * permission info by default. OPTIONAL code to add
         * azn_init_set_perminfo_attrs attribute to initdata
         * controls which permission info attributes are returned
         * by this call.
         */
        {
            azn_attrlist_h_t perminfo = AZN_C_INVALID_HANDLE;

            /* Perform extended authorization check */
            status = azn_decision_access_allowed_ext(creds,
                                                    obj_name,
                                                    operation,
                                                    AZN_C_INVALID_HANDLE,
                                                    &permitted,
                                                    &perminfo);
            check_status("azn_decision_access_allowed_ext", status);

            /* Dump the permission info attributes if any. */
            if (status == AZN_S_COMPLETE && perminfo != AZN_C_INVALID_HANDLE)
                attrlist_dump(perminfo);

            azn_attrlist_delete(&perminfo);
        }

#endif /* OPTIONAL_EXAMPLE_API */


        printf("\n\nResult: ");
        if ( permitted == AZN_C_PERMITTED)
            printf("Permitted.\n\n");
        else
            printf("Not permitted.\n\n");


#if OPTIONAL_EXAMPLE_API
        /*
         * An example calling sequence for azn_entitlement_get_entitlements
         * using the PD protected object entitlements service.
         * Not required for the authzn_demo.
         */
        {
            azn_attrlist_h_t ent_attrs = AZN_C_INVALID_HANDLE;
            azn_attrlist_h_t pobj_list = AZN_C_INVALID_HANDLE;
            unsigned int nvals = 0, i;
            azn_string_t tmpstr = NULL;

            azn_attrlist_create(&ent_attrs);

            status = azn_attrlist_add_entry(ent_attrs,
                                        azn_ent_svc_pd_pobj_path,
                                        obj_name);
            check_status("add attribute ent pobj path", status);
            if (status != AZN_S_COMPLETE)
            return(status);

            status = azn_attrlist_add_entry(ent_attrs,
                                        azn_ent_svc_pd_pobj_reqd_ops,
                                        operation);
            check_status("add attribute ent reqd ops", status);
            if (status != AZN_S_COMPLETE)
            return(status);

            /* retrieve the entitlements */
            status = azn_entitlement_get_entitlements(creds,
                                                    NULL,
                                                    ent_attrs,
                                                    &pobj_list);
            check_status("azn_entitlement_get_entitlements", status);
            if (status != AZN_S_COMPLETE) {
                fprintf(stderr, "Could get PD PObj entitlements!\n\n");
                azn_attrlist_delete(&ent_attrs);
                continue;
            }


            /* check that the entitlements list returned is valid. */
            status = azn_attrlist_name_get_num(pobj_list,
                                               azn_ent_svc_pd_pobj_matches,
                                               &nvals);
            if (status != AZN_S_COMPLETE) {
                if (azn_error_major(status) == AZN_S_INVALID_ATTRLIST_HDL)
                    fprintf(stderr, "PD PObj returned no entitlements.\n\n");
                else
                    check_status("get num of entitlements returned", status);

                azn_attrlist_delete(&ent_attrs);
                continue;
            }

            if (nvals == 0) {
                fprintf(stderr, "PD PObj entitlements count is 0.\n\n");
                azn_attrlist_delete(&ent_attrs);
                azn_attrlist_delete(&pobj_list);
                continue;
            }

            /* print the returned entitlements out */
            for (i = 0; i < nvals; i++) {

                /* get the supplied object name */
                status = azn_attrlist_get_entry_string_value(pobj_list,
                                                azn_ent_svc_pd_pobj_matches,
                                                i,
                                                &tmpstr);
                check_status("get entitlements entry", status);

                if (tmpstr != NULL) {
                    fprintf(stderr, "Entry %d [%s].\n", i, tmpstr);
                    azn_release_string(&tmpstr);
                }
                else
                    fprintf(stderr, "Entry %d is NULL!\n", i);
            }

            azn_attrlist_delete(&ent_attrs);
            azn_attrlist_delete(&pobj_list);
        }
#endif

        status = azn_creds_delete(&creds);
    }

    status = azn_shutdown();
    check_status("azn_shutdown", status);

    printf("\n\n");

    return 0;
}
void attrlist_dump(azn_attrlist_h_t perminfo)
{
        azn_string_t*               attr_names;
        int                         i ,j;
        unsigned int                count;
        azn_status_t                aznStatus;
        azn_buffer_t                buffer_value;
        azn_string_t                str_value;
        azn_ulong_t                 ulong_value;

        aznStatus = azn_attrlist_get_names(perminfo, &attr_names);
        check_status("azn_attrlist_get_names", aznStatus);
        if(attr_names){
            for (i = 0; attr_names[i] != NULL ;i++){
                aznStatus = azn_attrlist_name_get_num(perminfo,
                                attr_names[i],  &count);
                check_status("azn_attrlist_name_get_num", aznStatus);

                printf("\n-----------------------------\n");
                printf("Attr Name = %s\n" , attr_names[i]);
                printf("Values are : \n");
                for(j=0;j<count;j++){
                    aznStatus = azn_attrlist_get_entry_string_value(perminfo,attr_names[i],j,&str_value);
                    if ( aznStatus == AZN_S_COMPLETE ) {
                        printf("%s\n", str_value);
                        azn_release_string(&str_value);
                    }
                    else {
                        aznStatus = azn_attrlist_get_entry_buffer_value(perminfo,attr_names[i],j,&buffer_value);
                        if (aznStatus == AZN_S_COMPLETE ) {
                            printf("0x%08x\n" , buffer_value);
                            azn_release_buffer(&buffer_value);
                        }
                        else {
                            aznStatus = azn_attrlist_get_entry_ulong_value(perminfo,attr_names[i],j,&ulong_value);
                            if (aznStatus == AZN_S_COMPLETE )
                                printf("%d\n" , ulong_value);
                        }

                    }
                }
            }
       }
}
