/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2000, 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (c) 1998, 1999 DASCOM Inc.  All Rights Reserved.
 *
 * FILENAME
 *      aznutils.h
 *
 * DESCRIPTION
 *      Utility function supplement to the Auth API. These
 *      functions perform some commonly required tasks for
 *      Policy Directr Auth API clients.
 */

#ifndef _AZN_UTIL_H
#define _AZN_UTIL_H

#include <ogauthzn.h>


#ifdef __cplusplus
extern "C" {
#endif


/**********************************************************************
 * Utility Function Declarations
 **********************************************************************/

/*
 * FUNCTION NAME
 *      azn_util_password_authenticate
 *
 * DESCRIPTION
 *      Performs a login for a username/password pair and returns
 *      authentication information if login was successful. The
 *      means of authentication is selected automatically according
 *      to the underlying authentication mechanism that the auth api
 *      was installed with. eg. URAF or LDAP. For LDAP Auth API
 *      authentication the azn_initialize() call must have
 *      succeeded.  The 'mechanism_id' and 'authinfo' returned
 *      can then be addended with principal specific data and passed
 *      into the azn_get_creds() call.
 *
 * ARGUMENTS
 *      [in]  principal_name    Name of the principal to login as.
 *                              This will be a DN string if LDAP
 *                              authentication is in operation.
 *      [in]  password          Password for the user.
 *      [out] mechanism_id      String identifying authentication mechanism
 *                              that the user was authenticated with.
 *      [out] authinfo          Address of a pointer in which will be
 *                              returned the authentication info of the
 *                              user if authentication was successful.
 *
 * RETURN VALUE
 *      Returns AZN_S_COMPLETE on success, error code on failure
 *      On return the mechanism ID will be set to the corresponding
 *      authinfo type string if authentication succeeded. On failure
 *      both mechanism_id and authinfo will be NULL.
 *
 *
 * NOTES
 *      The 'mechanism_id' string is allocated by the utility function
 *      and must be freed using azn_util_release_string() when no longer
 *      needed. The authinfo is a malloc'd structure and should must
 *      also be freed by the caller.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_util_password_authenticate(
    const azn_string_t  principal_name,
    const azn_string_t  password,
    azn_string_t        *mechanism_id,
    azn_buffer_t        *authinfo
);


/*
 * FUNCTION NAME
 *      azn_util_errcode
 *
 * DESCRIPTION
 *      Builds an azn_status_t error code from a major and minor
 *      status.
 *
 * ARGUMENTS
 *      [in]  major             The major component of the error status.
 *      [in]  minor             The minor component of the error status.
 *
 * RETURN VALUE
 *      Returns complete azn_status_t error code.
 *
 * NOTES
 *
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_util_errcode(
    const unsigned int  major,
    const unsigned int  minor
);


/*
 * FUNCTION NAME
 *      azn_util_handle_is_valid
 *
 * DESCRIPTION
 *      Test the given handle for validity. This is used to determine
 *      whether a given handle actually references real data. Can be
 *      used by aznAPI and aznAPI service programmers to check a handle
 *      passed to it before attempting to work with it.
 *
 * ARGUMENTS
 *      [in]  handle    the handle to test for validity.
 *
 * RETURN VALUE
 *      TRUE if the handle is valid or FALSE if it is an invalid handle.
 *
 */
AZN_DECLSPEC
azn_boolean_t
AZN_CALLTYPE
azn_util_handle_is_valid(
    long handle
);



/*
 * FUNCTION NAME
 *      azn_util_password_change
 *
 * DESCRIPTION
 *      Changes the password for a username pair.
 *      The method of changing password is selected automatically
 *      according to the underlying authentication mechanism that
 *      the auth api was installed with. eg. URAF or LDAP.
 *      For LDAP Auth API the azn_util_registry_init() call must have
 *      been succeeded.
 *
 * ARGUMENTS
 *      [in]  principal_name    Name of the principal who's password
 *                              is to change.
 *                              This will be a DN string if LDAP
 *                              authentication is in operation.  However
 *                              it will convert it to a DN if it is not.
 *      [in]  password          New password for the principal.
 *
 * RETURN VALUE
 *      Returns AZN_S_COMPLETE on success, error code on failure
 *
 * NOTES
 *
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_util_password_change(
    const azn_string_t  principal_name,
    const azn_string_t  password
);


#ifdef __cplusplus
}
#endif


#endif /* _AZN_UTIL_H */
